@@ -1,14 +1,15 @@
 # 2016. 1. 10 by Hans Roh hansroh@gmail.com
 
-__version__ = "0.7.6.19"
+__version__ = "0.7.6.23"
 version_info = tuple (map (lambda x: not x.isdigit () and x or int (x),  __version__.split (".")))
-
+import threading
 from . import lifetime, queue, request_builder, response_builder, stubproxy
 from .lib import logger as logger_f
 from .lib.athreads import trigger
 from .client import socketpool
 from .dbapi import dbpool
-from .client import adns
+from .client import adns, asynconnect
+from .lib.athreads.fifo import await_fifo
 from . import client, dbapi
 from aquests.protocols.dns.asyndns import async_dns
 from .protocols.http import localstorage as ls
@@ -73,6 +74,7 @@ _timeout = 10
 _max_conns = 0
 _bytesrecv = 0
 _allow_redirects = True
+_force_h1 = False
 	
 def configure (
 	workers = 1, 
@@ -85,7 +87,7 @@ def configure (
 	allow_redirects = True,
 	qrandom = False
 ):
-	global _logger, _cb_gateway, _concurrent, _initialized, _timeout, _workers, _que, _allow_redirects
+	global _logger, _cb_gateway, _concurrent, _initialized, _timeout, _workers, _que, _allow_redirects, _force_h1
 	
 	if logger is None:
 		logger = logger_f.screen_logger ()
@@ -97,7 +99,11 @@ def configure (
 		_que = queue.Queue ()
 	
 	_allow_redirects = allow_redirects
-	request_handler.RequestHandler.FORCE_HTTP_11 = force_http1	
+	_force_h1 = request_handler.RequestHandler.FORCE_HTTP_11 = force_http1	
+	if not _force_h1:
+		asynconnect.AsynConnect.fifo_class = await_fifo
+		asynconnect.AsynSSLConnect.fifo_class = await_fifo
+		
 	http2.MAX_HTTP2_CONCURRENT_STREAMS = http2_constreams
 	_workers = workers
 	_concurrent = workers
@@ -153,7 +159,7 @@ def handle_status_3xx (response):
 	_que.first (response.request, 0)
 	
 def _request_finished (handler):
-	global _cb_gateway, _currents, _concurrent, _finished_total, _logger, _bytesrecv
+	global _cb_gateway, _currents, _concurrent, _finished_total, _logger, _bytesrecv,_force_h1
 	
 	if isinstance (handler, dbo_request.Request):
 		response = handler
@@ -161,33 +167,31 @@ def _request_finished (handler):
 		
 	else:
 		response = response_builder.HTTPResponse (handler)
-		_currents.pop (response.meta ['req_id'])		
-		
+		_currents.pop (response.meta ['req_id'])				
 		for handle_func in (handle_status_401, handle_status_3xx):
 			response = handle_func (response)
-			if not response:
-				return
-
+			if not response:			
+				return		
 	_finished_total += 1
 	response.logger = _logger
 	_bytesrecv += len (response.content)
 	callback = response.meta ['req_callback'] or _cb_gateway
 	try:
 		callback (response)
-	except:
+	except:		
 		_logger.trace ()	
 	
-	#if qsize ():
-		#_req ()
+	qsize () and _req ()
 		
 def _req ():
 	global _que, _logger, _currents, _request_total
-	args = _que.get ()	
+	args = _que.get ()
+	
 	_request_total += 1
 	_is_request = False
 	_is_db = False
 	_method = None
-	
+
 	if type (args) is not tuple:
 		req = args
 		meta = req.meta
@@ -209,7 +213,7 @@ def _req ():
 		req.set_callback (_request_finished)
 		asyncon.execute (req)
 		
-	else:	
+	else:
 		if not _is_request:
 			method, url, params, auth, headers, meta, proxy = args
 			asyncon = socketpool.get (url)
@@ -221,8 +225,9 @@ def _req ():
 			asyncon = socketpool.get (req.uri)		
 		_currents [meta ['req_id']] = [0, req.uri]
 		
+		asyncon.prevent_recursion = True
 		handler = req.handler (asyncon, req, _request_finished)
-		if asyncon.get_proto () and asyncon.isconnected ():
+		if asyncon.get_proto () and asyncon.isconnected ():			
 			asyncon.handler.handle_request (handler)
 		else:
 			handler.handle_request ()	
@@ -251,19 +256,20 @@ def concurrent ():
 	return _concurrent
 
 def fetchall ():
-	global _workers, _logger, _que, _timeout, _max_conns, _bytesrecv, _concurrent, _finished_total, _max_conns
+	global _workers, _logger, _que, _timeout, _max_conns, _bytesrecv, _concurrent, _finished_total, _max_conns	
 	
 	if not _initialized:
 		configure ()
 	
 	_fetch_started = timeit.default_timer ()
 	
-	if not request_handler.RequestHandler.FORCE_HTTP_11 and http2.MAX_HTTP2_CONCURRENT_STREAMS > 1:
-		# create initail workers	
-		_logger ("try to create http2 connection pool", "info")
-		target_socks = min (_workers, qsize ())
-		for i in range (target_socks):
-			_req ()	
+	# create initail workers	
+	_logger ("creating connection pool", "info")
+	target_socks = min (_workers, qsize ())
+	for i in range (target_socks):
+		_req ()
+			
+	if not request_handler.RequestHandler.FORCE_HTTP_11 and http2.MAX_HTTP2_CONCURRENT_STREAMS > 1:		
 		# wait all availabale	
 		while 1:
 			lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)
@@ -276,22 +282,22 @@ def fetchall ():
 		
 	# now starting
 	while qsize () or _currents:
-		#print ('---', _concurrent, len (_currents), mapsize (), qsize ())
 		while _concurrent > min (len (_currents), mapsize ()) and qsize ():
-			_req ()
-			_max_conns = max (_max_conns, mapsize ())		
-		lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)
+			_req ()			
+		_max_conns = max (_max_conns, mapsize ())	
+		#print ('---', _concurrent, len (_currents), mapsize (), qsize ())
+		lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)			
 		
 	_duration = timeit.default_timer () - _fetch_started	
 	socketpool.cleanup ()
 	dbpool.cleanup ()
 	
 	_logger.log ("* %d tasks during %1.2f sec (%1.2f tasks/s), recieved %d bytes (%d bytes/s), max %d conns" % (
-			_finished_total, _duration, 
-			_finished_total / _duration, 
-			_bytesrecv,
-			_bytesrecv / _duration,
-			_max_conns
+		_finished_total, _duration, 
+		_finished_total / _duration, 
+		_bytesrecv,
+		_bytesrecv / _duration,
+		_max_conns
 		)
 	)
 
@@ -303,7 +309,7 @@ def suspend (timeout):
 	time.sleep (a)
 
 _dns_reqs = 0
-def _add (method, url, params = None, auth = None, headers = {}, callback = None, meta = {}, proxy = None):	
+def _add (method, url, params = None, auth = None, headers = {}, callback = None, meta = None, proxy = None):	
 	global _que, _initialized, _dns_query_req, _dns_reqs
 	
 	def dns_result (answer = None):
@@ -313,11 +319,14 @@ def _add (method, url, params = None, auth = None, headers = {}, callback = None
 	if not _initialized:		
 		configure ()
 		
-	if not meta: meta = {}
+	if not meta: 
+		meta = {}
+
 	meta ['req_id'] = _que.req_id
 	meta ['req_method'] = method
 	meta ['req_callback'] = callback
 	host = urlparse (url) [1].split (":")[0]
+	
 	# DNS query for caching and massive
 	if not lifetime._polling:	
 		if host not in _dns_query_req:
