@@ -30,6 +30,8 @@ class AsynConnect (asynchat.async_chat):
 	request_count = 0
 	active = 0
 	fifo_class = deque
+	prevent_recursion = False
+	unused_error_port = 44413
 	
 	def __init__ (self, address, lock = None, logger = None):
 		self.address = address
@@ -95,7 +97,7 @@ class AsynConnect (asynchat.async_chat):
 		
 		handler, self.handller = self.handler, None
 		keep_active = False			
-		try: 
+		try:
 			keep_active = handler.connection_closed (self.errcode, self.errmsg)
 		except: 
 			self.trace ()
@@ -142,7 +144,8 @@ class AsynConnect (asynchat.async_chat):
 		self.__history = []
 		self._proto = None
 		self._handshaking = False
-		self._handshaked = False		
+		self._handshaked = False
+		self._dns_error = False
 		
 		self.established = False		
 		self.upgraded = False		
@@ -233,34 +236,34 @@ class AsynConnect (asynchat.async_chat):
 		self.set_socket (sock)
 			
 	def connect (self):
-		if DEBUG:
-			self.logger ('query DNS {rid:%s} %s' % (self.handler.request.meta.get ('req_id', -1), self.handler.request.uri), 'debug')
 		if adns.query:
 			adns.query (self.address [0], "A", callback = self.continue_connect)
-		else:
-			# no adns query
+		else:			
 			self.continue_connect (True)
 		
 	def continue_connect (self, answer = None):		
-		answer = answer or adns.get (self.address [0], "A")
-		ipaddr = answer and answer [-1]["data"] or None
-		if DEBUG:
-			self.logger ('got DNS {rid:%s} %s %s' % (self.handler.request.meta.get ('req_id', -1), res, self.handler.request.uri), 'debug')
-
-		if not ipaddr:			
-			# for perventing recursion, check already close			
-			return self.handle_close (704)
-		
 		self.initialize_connection ()
-		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)		
-		try:
-			if not adns.query:				
-				asynchat.async_chat.connect (self, self.address)				
-			else:	
-				asynchat.async_chat.connect (self, (ipaddr, self.address [1]))
-									
-		except:	
-			self.handle_error (714)
+		
+		if not adns.query:
+			self.create_socket (socket.AF_INET, socket.SOCK_STREAM)		
+			try: asynchat.async_chat.connect (self, self.address)
+			except:	self.handle_error (714)
+			return
+				
+		answer = answer or adns.get (self.address [0], "A")
+		ipaddr = answer and answer [-1]["data"] or None		
+		port = self.address [1]
+		
+		if not ipaddr:
+			if not self.prevent_recursion:
+				return self.handle_close (704)					
+			ipaddr = "127.0.0.1"
+			port = self.unused_error_port
+			self._dns_error = True
+												
+		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)
+		try: asynchat.async_chat.connect (self, (ipaddr, port))									
+		except:	self.handle_error (714)
 		
 	def recv (self, buffer_size):
 		try:
@@ -311,25 +314,37 @@ class AsynConnect (asynchat.async_chat):
 		self.zombie_timeout = timeout
 		
 	def handle_connect (self):
+		if self._dns_error:
+			# oops, used, really?
+			self.unused_error_port += 1
+			return self.handle_close (704, "DNS Not Found")
+			
 		if hasattr (self.handler, "has_been_connected"):		
 			self.handler.has_been_connected ()
-			
+	
+	def handle_dns_error (self):
+		self._dns_error = False
+		self.handle_close (704, "DNS Not Found")
+					
 	def handle_expt (self):
-		#self.logger ("socket panic", "fail")
+		if self._dns_error:
+			return self.handle_dns_error ()			
 		self.handle_close (703)
 	
 	def handle_error (self, code = 701):
+		if self._dns_error:
+			return self.handle_dns_error ()			
 		self.trace ()
 		self.handle_close(code)
 	
-	def handle_timeout (self):
-		#self.log ("socket timeout", "fail")
+	def handle_timeout (self):		
 		self.handle_close (702)
 		
 	def handle_expt_event(self):
+		if self._dns_error:
+			return self.handle_dns_error ()	
 		err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
 		if err != 0:
-			#self.log ("Socket Error %d Occurred" % err, "warn")
 			self.handle_close (703, "Socket %d Error" % err)
 		else:
 			self.handle_expt ()
@@ -433,7 +448,7 @@ class AsynSSLConnect (AsynConnect):
 		if not self._handshaking:
 			err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
 			if err != 0:
-				raise socket.error(err, _strerror(err))
+				raise socket.error(err, os.strerror(err))
 				
 			ssl_context = create_urllib3_context(ssl_version=resolve_ssl_version(None), cert_reqs=resolve_cert_reqs(None))
 			if self.ac_negotiate_http2:
@@ -472,7 +487,7 @@ class AsynSSLConnect (AsynConnect):
 	def recv (self, buffer_size):
 		if self._closed:
 			# usually handshaking failure, already handled exception
-			return
+			return b''
 			
 		try:
 			data = self.socket.recv (buffer_size)			
