@@ -47,7 +47,7 @@ except ImportError:
 	from email.utils import parseaddr
 from email.base64mime import body_encode as encode_base64
 from . import composer
-from aquests.lib import producers
+from ...lib import producers
 
 OLDSTYLE_AUTH = re.compile(r"auth=(.*)", re.I)
 FEATURE = re.compile (r'(?P<feature>[A-Za-z0-9][A-Za-z0-9\-]*)')
@@ -427,12 +427,14 @@ if __name__ == "__main__":
 	from skitai import lifetime
 	
 	log = logger.screen_logger ()
-	m = composer.Composer ("SMTP TEST", "@gmail.com", "@gmail.com")
-	m.add_text ("Hello World<div><img src='cid:A'></div>", "text/html")
+	m = composer.Composer ("smtp.gmail.com:587", "", "")
+	m.add_content ("Hello World<div><img src='cid:A'></div>", "text/html")
 	#m.add_attachment (r"1.png", cid="A")
 	
 	for i in range (1):
 		SMTP_SSL (m, log)
 		
 	asyncore.loop ()
+	
+	
 
